/*****************************************************************************
 *
 * FILE:	basilisk.h
 * DESCRIPTION:	simple HTML authoring library header
 * DATE:	Sun, Mar 31 2002
 * AUTHOR:	Kouichi ABE (WALL) / 
 * E-MAIL:	kouichi@MysticWALL.COM
 * URL:		http://www.MysticWALL.COM/cover.html
 * COPYRIGHT:	(C) 2002 졿Kouichi ABE (WALL), All rights reserved.
 * $Id: basilisk.h,v 1.2 2002/04/04 05:21:54 kouichi Exp $
 *
 *****************************************************************************/

#ifndef	_basilisk_h
#define	_basilisk_h	1

#define	BASILISK_VERSION_NUMBER	20020331
#define	BASILISK_VERSION_TEXT	"Basilisk/1.0(2002/03/31)"

/* useful macros */
#ifndef	EOL
#define	EOL	'\0'
#endif	/* EOL */
#ifndef	CR
#define	CR	'\015'
#endif	/* CR */
#ifndef	LF
#define	LF	'\012'
#endif	/* LF */
#ifndef	CRLF
#define	CRLF	"\015\012"
#endif	/* CRLF */
#ifndef	SPC
#define	SPC	'\040'
#endif	/* SPC */
#ifndef	TAB
#define	TAB	'\011'
#endif	/* TAB */


#ifndef	_STRING_T
#define	_STRING_T	1
typedef char *	String;
#endif	/* _STRING_T */

#ifndef	_BOOL_T
#define	_BOOL_T	1
typedef enum { FALSE, TRUE } bool_t;
#endif	/* _BOOL_T */


typedef char *		Length;
typedef unsigned int	Pixels;

typedef enum {	/* HTML3.2(headings,p,div) */
  HALIGN_NONE,
  HALIGN_LEFT,
  HALIGN_CENTER,
  HALIGN_RIGHT
} halign_t;

typedef enum {	/* HTML2.0(img, tr) */
  VALIGN_NONE,
  VALIGN_TOP,
  VALIGN_MIDDLE,
  VALIGN_BOTTOM
} valign_t;

typedef enum {	/* HTML3.2(caption) */
  CALIGN_NONE,
  CALIGN_TOP,
  CALIGN_BOTTOM
} calign_t;

typedef enum {	/* HTML3.2(img, input.image) */
  ALIGN_NONE,
  ALIGN_TOP,
  ALIGN_MIDDLE,
  ALIGN_BOTTOM,
  ALIGN_LEFT,
  ALIGN_RIGHT
} align_t;

typedef enum {	/* HTML3.2(ul) */
  BULLET_NONE,	/* default */
  BULLET_DISC,
  BULLET_SQUARE,
  BULLET_CIRCLE
} bullet_t;

typedef enum {	/* HTML3.2(ol) */
  NUMBERING_NONE,
  NUMBERING_ARABIC_NUMBER,
  NUMBERING_LOWER_ALPHA,
  NUMBERING_UPPER_ALPHA,
  NUMBERING_LOWER_ROMAN,
  NUMBERING_UPPER_ROMAN
} numbering_t;

typedef enum {	/* HTML document types */
  HTML_NONE,
  HTML2,
  HTML3,
  HTML4,
  HTML4Strict,
  HTML4Frame,
  HTML4Trans
} html_t;

typedef enum {
  METHOD_GET,
  METHOD_POST
} method_t;

typedef struct _Attrib {
  union {
    struct {
      String	bgcolor;	/* background color */
      String	text;		/* text color */
      String	link;		/* unvisited links color */
      String	vlink;		/* visited links color */
      String	alink;		/* highlight color */
      String	background;	/* image URL for document */
    } body;
    struct {
      bullet_t	bullet;		/* bullet style */
      bool_t	compact;	/* reduced interitem spacing */
    } ul;
    struct {
      numbering_t	numbering;	/* numbering style */
      unsigned int	start;		/* starting sequence */
      bool_t		compact;	/* reduced interitem spacing */
    } ol;
    struct {
      halign_t	halign;
      Pixels	size;
      Length	width;
      bool_t	noshade;
    } hr;
    struct {
      Pixels	border;		/* controls frame width around table */
      Length	width;		/* table width relative to window */
      Pixels	cellspacing;	/* spacing between cells */
      Pixels	cellpadding;	/* spacing within cells */
      halign_t	halign;		/* table position relative to window */
    } table;
    struct {
      bool_t		nowrap;	/* suppress word wrap */
      unsigned int	rowspan;/* number of rows spanned by cell */
      unsigned int	colspan;/* number of cols spanned by cell */
      halign_t		halign;	/* horizontal alignment in cells */
      valign_t		valign;	/* vertical alignment in cells */
      Pixels		width;	/* suggested width for cell */
      Pixels		height;	/* suggested height for cell */
    } cell;
  } x;
} Attrib;

typedef struct _html {
  html_t	version;
  union {
    struct {
      void	(*start_html)(String);
      void	(*end_html)(void);
      /**/
      void	(*h1)(String);
      void	(*h2)(String);
      void	(*h3)(String);
      void	(*h4)(String);
      void	(*h5)(String);
      void	(*h6)(String);
      /**/
      struct _block {
	void	(*begin)(void);
	void	(*end)(void);
      } block;
      struct _block	p;
      struct _block	pre;
      struct _block	address;
      /**/
      void	(*strong)(String);
      /**/
      struct {
	void	(*href)(String, String);
	void	(*name)(String, String);
      } a;
      /**/
      void	(*br)(void);
      /**/
      void	(*hr)(void);
      /**/
      void	(*img)(String, String, align_t);
      /**/
      struct {
	void	(*start)(String, method_t);
	void	(*end)(void);
      } form;
      struct {
	void	(*text)(String, String, unsigned int, unsigned int);
	void	(*password)(String, String, unsigned int, unsigned int);
	void	(*checkbox)(String, String, bool_t);
	void	(*radio)(String, String, bool_t);
	void	(*image)(String, String);
	void	(*hidden)(String, String);
	void	(*submit)(String, String);
	void	(*reset)(String, String);
      } input;
      struct {
	void	(*start)(String, unsigned int, bool_t);
	void	(*end)(void);
	void	(*option)(String, bool_t);
      } select;
      struct {
	void	(*start)(String, unsigned int, unsigned int);
	void	(*end)(void);
      } textarea;
      /**/
      void	(*print)(String);
      int	(*printf)();
      /**/
      void	(*done)(void);
    } HTML20;

    struct {
      void	(*start_html)(String);
      void	(*end_html)(void);
      /**/
      struct {
	void	(*start)(Attrib *);
	void	(*end)(void);
	void	(*clear)(Attrib *);
      } body;
      /**/
      void	(*h1)(String, halign_t);
      void	(*h2)(String, halign_t);
      void	(*h3)(String, halign_t);
      void	(*h4)(String, halign_t);
      void	(*h5)(String, halign_t);
      void	(*h6)(String, halign_t);
      /**/
      struct _block1 {
	void	(*begin)(void);
	void	(*end)(void);
      } address, pre, center;
      /**/
      struct _block2 {
	void	(*start)(halign_t);
	void	(*end)(void);
      } p, div;
      /**/
      struct _list {
	void	(*start)(Attrib *);
	void	(*end)(void);
	void	(*clear)(Attrib *);
      } ul, ol;
      void	(*li)(String);
      /**/
      struct {
	void	(*href)(String, String);
	void	(*name)(String, String);
      } a;
      /**/
      void	(*strong)(String);
      void	(*font)(String, String);
      /**/
      void	(*br)(void);
      /**/
      struct {
	void	(*start)(Attrib *);
	void	(*clear)(Attrib *);
      } hr;
      /**/
      void	(*img)(String, String, Pixels, Pixels, Pixels, align_t);
      /**/
      struct {
	void	(*start)(String, method_t);
	void	(*end)(void);
      } form;
      struct {
	void	(*start)(String, method_t);
	void	(*end)(void);
      } multipart_form;
      struct {
	void	(*text)(String, String, unsigned int, unsigned int);
	void	(*password)(String, String, unsigned int, unsigned int);
	void	(*checkbox)(String, String, bool_t);
	void	(*radio)(String, String, bool_t);
	void	(*submit)(String, String);
	void	(*reset)(String, String);
	void	(*hidden)(String, String);
	void	(*image)(String, String, align_t);
	void	(*file)(String, String, unsigned int, unsigned int);
      } input;
      struct {
	void	(*start)(String, unsigned int, bool_t);
	void	(*end)(void);
	void	(*option)(String, bool_t);
      } select;
      struct _textarea {
	void	(*start)(String, unsigned int, unsigned int);
	void	(*end)(void);
      } textarea;
      /**/
      struct {
	void	(*start)(Attrib *);
	void	(*end)(void);
	void	(*clear)(Attrib *);
	void	(*caption)(String, calign_t);
      } table;
      struct {
	void	(*start)(halign_t, valign_t);
	void	(*end)(void);
      } tr;
      struct {
	void	(*start)(Attrib *);
	void	(*end)(void);
	void	(*clear)(Attrib *);
      } th, td;
      /**/
      void	(*print)(String);
      int	(*printf)();
      /**/
      void	(*done)(void);
    } HTML32;
  } ver;
} HTML;

/* useful macros */
#define	html2	ver.HTML20
#define	html3	ver.HTML32

#define	Body		body.start(NULL)
#define	H1(x)		h1((x),HALIGN_NONE)
#define	H2(x)		h2((x),HALIGN_NONE)
#define	H3(x)		h3((x),HALIGN_NONE)
#define	H4(x)		h4((x),HALIGN_NONE)
#define	H5(x)		h5((x),HALIGN_NONE)
#define	H6(x)		h6((x),HALIGN_NONE)
#define	P		p.start(HALIGN_NONE)
#define	Div		div.start(HALIGN_NONE)
#define	UL		ul.start(NULL)
#define	OL		ol.start(NULL)
#define	Hr		ol.start(NULL)
#define	Table		table.start(NULL)
#define	Caption(x)	table.caption((x),CALIGN_NONE)
#define	Tr		tr.start(HALIGN_NONE,VALIGN_NONE)
#define	Th		th.start(NULL)
#define	Td		td.start(NULL)


/*****************************************************************************
 *****************************************************************************/

extern HTML *	newHTML(html_t);

#endif	/* _basilisk_h */

