/*****************************************************************************
 *
 * FILE:	cockatrice.h
 * DESCRIPTION:	simple CGI library header
 * DATE:	Wed, Feb 14 2001
 * AUTHOR:	Kouichi ABE (WALL) / 
 * E-MAIL:	kouichi@MysticWALL.COM
 * URL:		http://www.MysticWALL.COM/cover.html
 * COPYRIGHT:	(C) 2001-2002 졿Kouichi ABE (WALL), All rights reserved.
 * $Id: cockatrice.h,v 1.21 2002/07/22 08:36:13 kouichi Exp $
 *
 *****************************************************************************/

#ifndef	_cockatrice_h
#define	_cockatrice_h	1

#define	COCKATRICE_VERSION_NUMBER	20020722
#define	COCKATRICE_VERSION_TEXT		"Cockatrice/1.0(2002/07/22)"

/* useful macros */
#ifndef	EOL
#define	EOL	'\0'
#endif	/* EOL */
#ifndef	CR
#define	CR	'\015'
#endif	/* CR */
#ifndef	LF
#define	LF	'\012'
#endif	/* LF */
#ifndef	CRLF
#define	CRLF	"\015\012"
#endif	/* CRLF */
#ifndef	SPC
#define	SPC	'\040'
#endif	/* SPC */
#ifndef	TAB
#define	TAB	'\011'
#endif	/* TAB */


#ifndef	_STRING_T
#define	_STRING_T	1
typedef char *	String;
#endif	/* _STRING_T */

#ifndef	_BOOL_T
#define	_BOOL_T	1
typedef enum { FALSE, TRUE } bool_t;
#endif	/* _BOOL_T */

#ifndef	_VLIST_T
#define	_VLIST_T	1
typedef struct _vlist {
  String		value;
  struct _vlist *	next;
} VList;
#endif	/* _VLIST_T */

typedef enum {	/* cockatrice error types */
  CGIERR_NO_ERROR = 0,
  CGIERR_ALREDY_MADE_OBJECT,
  CGIERR_NO_ENOUGH_MEMORY,
  CGIERR_NO_TEMPORARY_DIR,
  CGIERR_NO_REQUEST_METHOD,
  CGIERR_NO_QUERY_STRING,
  CGIERR_NO_CONTENT_LENGTH,
  CGIERR_ILLEGAL_METHOD_TYPE,
  CGIERR_ILLEGAL_MULTIPART_FORM,
  CGIERR_TMPFILE_OPEN_FAILURE,
  CGIERR_DB_OPEN_FAILURE,
  CGIERR_NO_SESSION_ID,
  CGIERR_EXPIRED_SESSION_ID,
  CGIERR_INVALID_SESSION_ID,
  CGIERR_SESSION_ID_DELETE_FAILURE,
  CGIERR_NO_CURRENT_TIME,
  CGIERR_INVALID_ARGUMENTS,
  CGIERR_OTHER_PROCESS_USING,
  CGIERR_INTERNAL_LIBRARY_ERROR,
  CGIERR_WILDCARD	/* use this macro in your program */
} cgierr_t;


/*****************************************************************************
 *****************************************************************************/

typedef char *		Length;
typedef int		Pixels;
typedef unsigned int	Number;

typedef enum {
  HTML4,
  HTML4_Strict,
  HTML4_Transitional,
  HTML4_Frameset
} dtd_t;

typedef enum {
  LINK_NONE,
  LINK_ALTERNATE,
  LINK_STYLESHEET,
  LINK_START,
  LINK_NEXT,
  LINK_PREV,
  LINK_CONTENTS,
  LINK_INDEX,
  LINK_GLOSSARY,
  LINK_COPYRIGHT,
  LINK_CHAPTER,
  LINK_SECTION,
  LINK_SUBSECTION,
  LINK_APPENDIX,
  LINK_HELP,
  LINK_BOOKMARK
} link_t;

typedef enum {
  MEDIA_NONE,
  MEDIA_ALL,
  MEDIA_AURAL,
  MEDIA_BRAILLE,
  MEDIA_EMBOSSED,
  MEDIA_HANDHELD,
  MEDIA_PRINT,
  MEDIA_PROJECTION,
  MEDIA_SCREEN,
  MEDIA_TTY,
  MEDIA_TV
} media_t;

typedef enum {
  ALIGN_NONE,
  ALIGN_LEFT,
  ALIGN_CENTER,
  ALIGN_RIGHT,
  ALIGN_JUSTIFY,
  ALIGN_TOP,
  ALIGN_MIDDLE,
  ALIGN_BOTTOM,
  ALIGN_CHAR,
  ALIGN_BASELINE
} align_t;

typedef enum {	/* for <BR> */
  CLEAR_NONE,
  CLEAR_LEFT,
  CLEAR_RIGHT,
  CLEAR_ALL
} clear_t;

typedef enum {	/* for <UL> */
  BULLET_NONE,
  BULLET_DISC,
  BULLET_CIRCLE,
  BULLET_SQUARE
} bullet_t;

typedef enum {	/* for <OL> */
  NUMBERING_NONE,
  NUMBERING_ARABIC_NUMBER,
  NUMBERING_LOWER_ALPHA,
  NUMBERING_UPPER_ALPHA,
  NUMBERING_LOWER_ROMAN,
  NUMBERING_UPPER_ROMAN
} numbering_t;

typedef enum {	/* for <LI> */
  STYLE_NONE,
  STYLE_DISC,
  STYLE_CIRCLE,
  STYLE_SQUARE,
  STYLE_ARABIC_NUMBER,
  STYLE_LOWER_ALPHA,
  STYLE_UPPER_ALPHA,
  STYLE_LOWER_ROMAN,
  STYLE_UPPER_ROMAN
} style_t;

typedef enum {	/* for <TABLE> */
  FRAME_VOID,
  FRAME_ABOVE,
  FRAME_BELOW,
  FRAME_HSIDES,
  FRAME_LHS,
  FRAME_RHS,
  FRAME_VSIDES,
  FRAME_BOX,
  FRAME_BORDER
} frame_t;

typedef enum {	/* for <TABLE> */
  RULES_NONE,
  RULES_GROUPS,
  RULES_ROWS,
  RULES_COLS,
  RULES_ALL
} rules_t;

typedef enum {	/* for <TH>,<TD> */
  SCOPE_NONE,
  SCOPE_ROW,
  SCOPE_COL,
  SCOPE_ROWGROUP,
  SCOPE_COLGROUP
} scope_t;

typedef enum {	/* for <IFRAME>, <FRAME> */
  SCROLLING_AUTO,
  SCROLLING_YES,
  SCROLLING_NO
} scrolling_t;

typedef enum {
  METHOD_GET,
  METHOD_POST
} method_t;

typedef enum {
  DIGEST_MD5,
  DIGEST_SHA1
} digest_t;

typedef struct {
  /* Document Type Definition */
  dtd_t	dtd;

  /*
   * Strict
   */
  /* HTML/HEAD */
  struct {
    void	(*begin)(void);
    void	(*end)(void);
    struct {
      String	lang;
      void	(*init)(void);
    } attr;
  } html, head;
  /* TITLE */
  void		(*title)(const String);	/* title */
  /* META */
  struct {
    void	(*name)(const String, const String);	/* name, content */
		/* http-equiv, content */
    void	(*http_equiv)(const String, const String);
    struct {
      String	lang;
      void	(*init)(void);
    } attr;
  } meta;
  /* LINK */
#define	LINK	link.output()
  struct {
    void	(*output)(void);
    struct {
      String	href;
      String	type;	/* mime-type */
      link_t	rel;
      link_t	rev;
      media_t	media;
      String	charset;
      void	(*init)(void);
    } attr;
  } link;
  /* BASE */
  void		(*base)(const String);	/* URI */
  /* STYLE */
  struct {
    void	(*begin)(const String, media_t);	/* mime-type, media */
    void	(*end)(void);
  } style;
  /* BODY */
  struct {
    void	(*begin)(void);
    void	(*end)(void);
    struct {
      struct {
	String	background;	/* URI */
	String	bgcolor;	/* color */
	String	text;		/* color */
	String	link;		/* color */
	String	vlink;		/* color */
	String	alink;		/* color */
      } trans;
      /* general attributes */
      String	id;
      String	class;
      String	lang;
      String	title;
      String	style;
      void	(*init)(void);
    } attr;
  } body;
  /* SPAN/H1..H6 */
#define	SPAN(x)	span.output((x))
#define	H1(x)	h1.output((x))
#define	H2(x)	h2.output((x))
#define	H3(x)	h3.output((x))
#define	H4(x)	h4.output((x))
#define	H5(x)	h5.output((x))
#define	H6(x)	h6.output((x))
  struct _headings {
    void	(*output)(const String);	/* text */
    struct {
      struct {
	align_t	align;	/* left|center|right|justify */
      } trans;
      /* general attributes */
      String	id;
      String	class;
      String	lang;
      String	title;
      String	style;
      void	(*init)(void);
    } attr;
  } span, h1, h2, h3, h4, h5, h6;
  /* DIV/P */
  struct _block {
    void	(*begin)(void);
    void	(*end)(void);
    struct {
      struct {
	align_t	align;	/* left|center|right|justify */
      } trans;
      /* general attributes */
      String	id;
      String	class;
      String	lang;
      String	title;
      String	style;
      void	(*init)(void);
    } attr;
  } div, p;
  /* ADDRESS */
  struct {
    void	(*begin)(void);
    void	(*end)(void);
    struct {
      String	id;
      String	class;
      String	lang;
      void	(*init)(void);
    } attr;
  } address;
  /* PRE */
  struct {
    void	(*begin)(void);
    void	(*end)(void);
    struct {
      String	id;
      String	class;
      String	lang;
      String	title;
      String	style;
      void	(*init)(void);
    } attr;
  } pre;
  /* BR */
#define	BR	br.output()
  struct {
    void	(*output)(void);
    struct {
      struct {
	clear_t	clear;	/* none|left|right|all */
      } trans;
      /* general attributes */
      String	id;
      String	class;
      String	title;
      String	style;
      void	(*init)(void);
    } attr;
  } br;
  /* EM/STRONG */
#define	EM(x)		em.output((x))	
#define	STRONG(x)	strong.output((x))	
  struct _modifier {
    void	(*output)(const String);
    struct {
      String	id;
      String	class;
      String	lang;
      String	title;
      String	style;
      void	(*init)(void);
    } attr;
  } em, strong;
  /* HR */
#define	HR	hr.output()
  struct {
    void	(*output)(void);
    struct {
      struct {	/* using in "HTML 4.01 Transitional" */
	align_t	align;	/* left|center|right */
	bool_t	noshade;
	Pixels	size;
	Length	width;
      } trans;
      /* general attributes */
      String	id;
      String	class;
      String	title;
      String	style;
      void	(*init)(void);
    } attr;
  } hr;
  /* IMG */
#define	IMG(a,b,c,d)	img.output((a),(b),(c),(d))
  struct {
    void	(*output)(const String, const String, Length, Length);
		/* src, alt, width, height */
    struct {
      struct {	/* using in "HTML 4.01 Transitional" */
	align_t	align;	/* top|middle|bottom|left|right */
	Length	border;
	Pixels	hspace;
	Pixels	vspace;
      } trans;
      /* general attributes */
      String	id;
      String	class;
      String	lang;
      String	title;
      String	style;
      void	(*init)(void);
    } attr;
  } img;
  /* UL */
  struct {
    void	(*begin)(void);
    void	(*end)(void);
    struct {
      struct {	/* using in "HTML 4.01 Transitional" */
	bullet_t	bullet;	/* disc|circle|square */
	bool_t		compact;
      } trans;
      /* general attributes */
      String	id;
      String	class;
      String	lang;
      String	title;
      String	style;
      void	(*init)(void);
    } attr;
  } ul;
  /* OL */
  struct {
    void	(*begin)(void);
    void	(*end)(void);
    struct {
      struct {	/* using in "HTML 4.01 Transitional" */
	numbering_t	numbering;	/* 1|a|A|i|I */
	int		start;
	bool_t		compact;
      } trans;
      /* general attributes */
      String	id;
      String	class;
      String	lang;
      String	title;
      String	style;
      void	(*init)(void);
    } attr;
  } ol;
  /* LI */
#define	LI	li.begin()
  struct {
    void	(*begin)(void);
    void	(*end)(void);
    struct {
      struct {	/* using in "HTML 4.01 Transitional" */
	style_t	style;	/* disc|circle|square|1|a|A|i|I */
	int	value;
      } trans;
      /* general attributes */
      String	id;
      String	class;
      String	lang;
      String	title;
      String	style;
      void	(*init)(void);
    } attr;
  } li;
  /* TABLE */
  struct {
    void	(*begin)(void);
    void	(*end)(void);
    struct {
      String	summary;
      Length	width;
      frame_t	frame;	/* void|above|below|hsides|lhs|rhs|vsides|box|border */
      rules_t	rules;	/* none|groups|rows|cols|all */
      Pixels	border;
      Length	cellspacing;
      Length	cellpadding;
      struct {	/* using in "HTML 4.01 Transitional" */
	align_t	align;	/* left|center|right */
	String	bgcolor;
      } trans;
      /* general attributes */
      String	id;
      String	class;
      String	lang;
      String	title;
      String	style;
      void	(*init)(void);
    } attr;
  } table;
  /* CAPTION */
#define	CAPTION(x)	caption.output((x))
  struct {
    void	(*output)(const String);	/* caption */
    struct {
      struct {	/* using in "HTML 4.01 Transitional" */
	align_t	align;	/* top|bottom|left|right */
      } trans;
      /* general attributes */
      String	id;
      String	class;
      String	lang;
      String	title;
      String	style;
      void	(*init)(void);
    } attr;
  } caption;
  /* TR */
  struct {
    void	(*begin)(void);
    void	(*end)(void);
    struct {
      align_t	align;	/* left|center|right|justify|char */
      align_t	valign;	/* top|middle|bottom|baseline */
      String	Char;
      Length	charoff;
      struct {	/* using in "HTML 4.01 Transitional" */
	String	bgcolor;
      } trans;
      /* general attributes */
      String	id;
      String	class;
      String	lang;
      String	title;
      String	style;
      void	(*init)(void);
    } attr;
  } tr;
  /* TH/TD */
  struct _cell {
    void	(*begin)(void);
    void	(*end)(void);
    struct {
      scope_t	scope;	/* row|col|rowgroup|colgroup */
      String	abbr;
      String	axis;
      int	rowspan;
      int	colspan;
      align_t	align;	/* left|center|right|justify|char */
      align_t	valign;	/* top|middle|bottom|baseline */
      String	Char;
      Length	charoff;
      struct {	/* using in "HTML 4.01 Transitional" */
	bool_t	nowrap;
	Pixels	width;
	Pixels	height;
	String	bgcolor;
      } trans;
      /* general attributes */
      String	id;
      String	class;
      String	lang;
      String	title;
      String	style;
      void	(*init)(void);
    } attr;
  } th, td;
  /* A */
  struct {
    void	(*href)(const String, const String);	/* URI, text */
    void	(*name)(const String, const String);	/* Name, text */
    struct {
      String	type;	/* mime-type */
      link_t	rel;
      link_t	rev;
      String	target;
      /* general attributes */
      String	id;
      String	class;
      String	lang;
      String	title;
      String	style;
      void	(*init)(void);
    } attr;
  } a;
  /* FORM */
  struct _form {
    void	(*begin)(const String, method_t);
    void	(*end)(void);
    struct {
      /* general attributes */
      String	id;
      String	class;
      String	lang;
      String	title;
      String	style;
      void	(*init)(void);
    } attr;
  } form, multipart_form;
  /* INPUT */
  struct {
    void	(*text)(const String, const String, Pixels, Pixels);
    void	(*password)(const String, const String, Pixels, Pixels);
    void	(*checkbox)(const String, const String, bool_t);
    void	(*radio)(const String, const String, bool_t);
    void	(*submit)(const String, const String);
    void	(*reset)(const String, const String);
    void	(*file)(const String, const String, Pixels, Pixels);
    void	(*hidden)(const String, const String);
    void	(*image)(const String, const String, const String);
    void	(*button)(const String, const String);
    struct {
      bool_t	readonly;	/* default: FALSE */
      bool_t	disabled;	/* default: FALSE */
      struct {	/* using in "HTML 4.01 Transitional" */
	align_t	align;	/* top|middle|bottom|left|right */
      } trans;
      /* general attributes */
      String	id;
      String	class;
      String	lang;
      String	title;
      String	style;
      void	(*init)(void);
    } attr;
  } input;
  /* SELECT */
  struct {
			/* name, size, multiple */
    void	(*begin)(const String, Pixels, bool_t);
    void	(*end)(void);
    struct {
      /* general attributes */
      String	id;
      String	class;
      String	lang;
      String	title;
      String	style;
      void	(*init)(void);
    } attr;
  } select;
  /* OPTION */
#define	OPTION(x,y,z)	option.output((x),(y),(z))
  struct {
			/* text, value, selected */
    void	(*output)(const String, const String, bool_t);
    struct {
      String	label;
      bool_t	disabled;
      /* general attributes */
      String	id;
      String	class;
      String	lang;
      String	title;
      String	style;
      void	(*init)(void);
    } attr;
  } option;
  /* TEXTAREA */
  struct {
			/* name, rows, cols */
    void	(*begin)(const String, Number, Number);
    void	(*end)(void);
    struct {
      bool_t	readonly;
      bool_t	disabled;
      /* general attributes */
      String	id;
      String	class;
      String	lang;
      String	title;
      String	style;
      void	(*init)(void);
    } attr;
  } textarea;

  /*
   * Transitional
   */
  /* FONT */
		/* text, size, color */
  void	(*font)(const String, const String, const String);
  /* CENTER */
  struct {
    void	(*begin)(void);
    void	(*end)(void);
  } center;
  /* IFRAME */
  struct {
    void	(*begin)(const String, Length, Length); /* src, width, height */
    void	(*end)(void);
    struct {
      scrolling_t	scrolling;	/* auto|yes|no */
      bool_t		frameborder;	/* 1|0 */
      Pixels		marginwidth;
      Pixels		marginheight;
      struct {
	align_t		align;	/* left|center|right|justify */
      } trans;
      void		(*init)(void);
    } attr;
  } iframe;

  /*
   * Frameset
   */
  /* FRAMESET */
  struct {
    void	(*begin)(Length, Length);
    void	(*end)(void);
  } frameset;

  /* FRAME */
#define	FRAME(x)	frame.output((x))
  struct {
    void	(*output)(const String);	/* src */
    struct {
      bool_t		noresize;
      scrolling_t	scrolling;	/* auto|yes|no */
      bool_t		frameborder;	/* 1|0 */
      Pixels		marginwidth;
      Pixels		marginheight;
      void		(*init)(void);
    } attr;
  } frame;

  /* NOFRAMES */
  struct {
    void	(*begin)(void);
    void	(*end)(void);
  } noframes;

  /* print procedures */
  void	(*print)(const String);
  void	(*printe)(const String);
  void	(*printf)();
} HTML;


/*****************************************************************************
 *****************************************************************************/

typedef struct _cgi {
  String	script_name;	/* CGI program name */
  /**/
  String	tmpdir;		/* safe temporary directory */
  /* parameter */
  String	(*param)(const String);
  VList *	(*params)(const String);
  /* cookie */
  String	(*cookie)(const String);
  struct {
    int		(*Name)(const String, const String);
    int		(*Comment)(const String, const String);
    int		(*Domain)(const String, const String);
    int		(*MaxAge)(const String, unsigned int);
    int		(*Path)(const String, const String);
    int		(*Secure)(const String, bool_t);
  } setCookie;
  /* session */
  struct {
    int		(*openDB)(const String);
    int		(*closeDB)(void);
#if	0
    int		(*changeDB)(const String);
#endif
    void	(*setExpires)(unsigned int);
    unsigned int	(*getExpires)(void);
    String	(*generateID)(void);
    String	(*generateID1)(const String);
    String	(*generateID2)(const String, const String);
    int		(*validateID)(const String);
    int		(*deleteID)(const String);
    int		(*countValidID)(void);
  } session;
  /* upload */
  struct {
    FILE *	(*open)(const String);
    int		(*store)(const String, const String);
    String	(*digest)(const String, digest_t);
  } upload;
  /* environment variables */
  String	(*getenv)(const String);
  struct {
    String		(*server_software)(void);
    String		(*server_name)(void);
    String		(*gateway_interface)(void);
    String		(*server_protocol)(void);
    unsigned int	(*server_port)(void);
    String		(*request_method)(void);
    String		(*script_name)(void);
    String		(*path_info)(void);
    String		(*path_translated)(void);
    String		(*remote_host)(void);
    String		(*remote_addr)(void);
    unsigned int	(*remote_port)(void);
    String		(*auth_type)(void);
    String		(*remote_user)(void);
    String		(*remote_ident)(void);
    String		(*accept)(void);
    String		(*accept_encoding)(void);
    String		(*accept_language)(void);
    String		(*user_agent)(void);
    String		(*referer)(void);
    String		(*document_root)(void);
    String		(*server_root)(void);
    String		(*server_admin)(void);
  } env;
  /* utilities */
  struct {
    /* lock/unlock */
    int		(*lock)(const String);
    int		(*unlock)(const String);
    /* escape/unescape */
    String	(*escape)(String);
    String	(*unescape)(String);
    /* md5/sha1 */
    String	(*digest)(String, digest_t);
  } util;
  /* date */
  struct {
    String	(*today)(void);	/* ex. 2002/02/02(Sat) */
    int		(*year)(void);	/* ex. 2002 */
    int		(*month)(void);	/* 1 .. 12 */
    int		(*day)(void);	/* 1 .. 31 */
    int		(*wday)(void);	/* 0:Sun .. 6:Sat */
    bool_t	(*isFestival)(int, int, int);
    int		(*getWDay)(int, int, int);	/* 0:Sun .. 6:Sat */
    String	(*time)(void);	/* ex. 02:22:22 */
  } date;
  /* check */
  struct {
    int		(*empty)(const String);
    int		(*digit)(const String);
    int		(*alphabet)(const String);
    int		(*username)(const String);
    int		(*hostname)(const String);
    int		(*domainname)(const String);
    int		(*email)(const String);
    int		(*range)(const String, int, int);
    int		(*ipaddr)(const String);
    int		(*ipaddr2)(const String);
    int		(*ipaddr3)(const String);
  } check;
  /* finish */
  void		(*done)(void);
  /* content-type */
  void		(*header)(const String);
  void		(*redirect)(const String);
  /* html authoring */
  HTML *	html;
} CGI;

/*****************************************************************************
 *****************************************************************************/

extern CGI *	newCGI(dtd_t);
extern cgierr_t	cgi_errno;
#if	DEBUG
extern FILE *	debugLog;
#endif	/* DEBUG */

#endif	/* _cockatrice_h */

