/*****************************************************************************
 *
 * FILE:	env.c
 * DESCRIPTION:	environment variable manipulator for simple CGI library
 * DATE:	Sun, Mar 17 2002
 * AUTHOR:	Kouichi ABE (WALL) / 
 * E-MAIL:	kouichi@MysticWALL.COM
 * URL:		http://www.MysticWALL.com/cover.html
 * COPYRIGHT:	(C) 2002 졿Kouichi ABE (WALL), All rights reserved.
 * $Id: env.c,v 1.3 2002/06/15 03:32:33 kouichi Exp $
 *
 *****************************************************************************/

#if	HAVE_CONFIG_H
#include "config.h"
#endif	/* HAVE_CONFIG_H */

#include <stdio.h>
#include <stdlib.h>
#if	HAVE_UNISTD_H
#include <unistd.h>
#endif	/* HAVE_UNISTD_H */
#include "cockatrice.h"
#include "functions.h"


/*****************************************************************************
 *
 *	definition of macros and structures
 *
 *****************************************************************************/
typedef struct _environment {
  String	server_software;
  String	server_name;
  String	gateway_interface;
  /**/
  String	server_protocol;
  unsigned int	server_port;
  /**/
  String	request_method;
  String	script_name;
  String	path_info;
  String	path_translated;
  String	remote_host;
  String	remote_addr;
  unsigned int	remote_port;
  String	auth_type;
  String	remote_user;
  String	remote_ident;
  /**/
  String	accept;
  String	accept_encoding;
  String	accept_language;
  String	user_agent;
  String	referer;
  /**/
  String	document_root;
  String	server_root;
  String	server_admin;
} Env;


/*****************************************************************************
 *
 *	declaration of local variables
 *
 *****************************************************************************/
static Env *	env;


/*****************************************************************************
 *
 *	definition of functions
 *
 *****************************************************************************/

/*
 * FUNCTION NAME:	envInit
 * DESCRIPTION:		initialize envrionment variable
 * ARGUMENTS:		none
 * RETURN VALUE:	Successful: 0, otherwise: -1
 */
int
envInit(void)
{
  String	p;	/* working pointer */

  env = (Env *)malloc(sizeof(Env));
  if (env == NULL) {
    cgi_errno = CGIERR_NO_ENOUGH_MEMORY;
    return -1;
  }

  env->server_software	= getenv("SERVER_SOFTWARE");
  env->server_name	= getenv("SERVER_NAME");
  env->gateway_interface= getenv("GATEWAY_INTERFACE");
  /**/
  env->server_protocol	= getenv("SERVER_PROTOCOL");

  p			= getenv("SERVER_PORT");
  env->server_port	= p == NULL ? 0 : (unsigned int)atoi(p);;
  /**/
  env->request_method	= getenv("REQUEST_METHOD");
  env->script_name	= getenv("SCRIPT_NAME");
  env->path_info	= getenv("PATH_INFO");
  env->path_translated	= getenv("PATH_TRANSLATED");
  env->remote_host	= getenv("REMOTE_HOST");
  env->remote_addr	= getenv("REMOTE_ADDR");

  p			= getenv("REMOTE_PORT");
  env->remote_port	= p == NULL ? 0 : (unsigned int)atoi(p);

  env->auth_type	= getenv("AUTH_TYPE");
  env->remote_user	= getenv("REMOTE_USER");
  env->remote_ident	= getenv("REMOTE_IDENT");
  /**/
  env->accept		= getenv("HTTP_ACCEPT");
  env->accept_encoding	= getenv("HTTP_ACCEPT_ENCODING");
  env->accept_language	= getenv("HTTP_ACCEPT_LANGUAGE");
  env->user_agent	= getenv("HTTP_USER_AGENT");
  env->referer		= getenv("HTTP_REFERER");
  /**/
  env->document_root	= getenv("DOCUMENT_ROOT");
  env->server_root	= getenv("SERVER_ROOT");
  env->server_admin	= getenv("SERVER_ADMIN");

  return 0;
}

/*
 * FUNCTION NAME:	envRelease
 * DESCRIPTION:		release memory area
 * ARGUMENTS:		none
 * RETURN VALUE:	none
 */
void
envRelease(void)
{
  free((Env *)env);
}


/*****************************************************************************
 *****************************************************************************/

String
envServerSoftware(void)
{
  return env->server_software;
}

String
envServerName(void)
{
  return env->server_name;
}

String
envGatewayInterface(void)
{
  return env->gateway_interface;
}

/**/

String
envServerProtocol(void)
{
  return env->server_protocol;
}

unsigned int
envServerPort(void)
{
  return env->server_port;
}

/**/

String
envRequestMethod(void)
{
  return env->request_method;
}

String
envScriptName(void)
{
  return env->script_name;
}

String
envPathInfo(void)
{
  return env->path_info;
}

String
envPathTranslated(void)
{
  return env->path_translated;
}

String
envRemoteHost(void)
{
  return env->remote_host;
}

String
envRemoteAddr(void)
{
  return env->remote_addr;
}

unsigned int
envRemotePort(void)
{
  return env->remote_port;
}

String
envAuthType(void)
{
  return env->auth_type;
}

String
envRemoteUser(void)
{
  return env->remote_user;
}

String
envRemoteIdent(void)
{
  return env->remote_ident;
}

/**/

String
envHttpAccept(void)
{
  return env->accept;
}

String
envHttpAcceptEncoding(void)
{
  return env->accept_encoding;
}

String
envHttpAcceptLanguage(void)
{
  return env->accept_language;
}

String
envHttpUserAgent(void)
{
  return env->user_agent;
}

String
envHttpReferer(void)
{
  return env->referer;
}

/**/

String
envDocumentRoot(void)
{
  return env->document_root;
}

String
envServerRoot(void)
{
  return env->server_root;
}

String
envServerAdmin(void)
{
  return env->server_admin;
}

